@@ -1,6 +1,6 @@
 /*
  * Copyright 2014-2016 Canonical Ltd.
- * Copyright 2021-2022 Robert Tari
+ * Copyright 2021-2023 Robert Tari
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3, as published
@@ -88,6 +88,9 @@ typedef struct
 
   gboolean caps_queried;
   gboolean actions_supported;
+  #ifdef LOMIRI_FEATURES_ENABLED
+  gboolean lomiri_snap_decisions_supported;
+  #endif
 
   GCancellable * cancellable;
   #ifdef LOMIRI_FEATURES_ENABLED
@@ -147,6 +150,19 @@ get_battery_power_level (IndicatorPowerDevice * battery)
   return ret;
 }
 
+static gdouble
+get_battery_power_percentage (IndicatorPowerDevice * battery)
+{
+  gdouble ret;
+
+  g_return_val_if_fail(battery != NULL, 0);
+  g_return_val_if_fail(indicator_power_device_get_kind(battery) == UP_DEVICE_KIND_BATTERY, 0);
+
+  ret = indicator_power_device_get_percentage(battery);
+
+  return ret;
+}
+
 /***
 ****  Sounds
 ***/
@@ -252,8 +268,8 @@ on_dismiss_clicked(NotifyNotification * nn        G_GNUC_UNUSED,
   /* no-op; libnotify warns if we have a NULL action callback */
 }
 
-static gboolean
-are_actions_supported(IndicatorPowerNotifier * self)
+static void
+ensure_caps_queried(IndicatorPowerNotifier * self)
 {
   priv_t * const p = get_priv(self);
 
@@ -263,22 +279,54 @@ are_actions_supported(IndicatorPowerNotifier * self)
       GList * caps;
       GList * l;
 
-      /* see if actions are supported */
+      #ifdef LOMIRI_FEATURES_ENABLED
+      gboolean lomiri_snap_decisions_supported = FALSE;
+      #endif
+
+      /* see if actions and snap decisions are supported */
       actions_supported = FALSE;
       caps = notify_get_server_caps();
-      for (l=caps; l!=NULL && !actions_supported; l=l->next)
+      for (l=caps; l!=NULL; l=l->next) {
         if (!g_strcmp0(l->data, "actions"))
           actions_supported = TRUE;
+        #ifdef LOMIRI_FEATURES_ENABLED
+        else if (!g_strcmp0(l->data, "x-lomiri-snap-decisions"))
+          lomiri_snap_decisions_supported = TRUE;
+        #endif
+      }
 
       p->actions_supported = actions_supported;
+      #ifdef LOMIRI_FEATURES_ENABLED
+      p->lomiri_snap_decisions_supported = lomiri_snap_decisions_supported;
+      #endif
       p->caps_queried = TRUE;
 
       g_list_free_full(caps, g_free);
     }
+}
+
+static gboolean
+are_actions_supported(IndicatorPowerNotifier * self)
+{
+  priv_t * const p = get_priv(self);
+
+  ensure_caps_queried(self);
 
   return p->actions_supported;
 }
 
+#ifdef LOMIRI_FEATURES_ENABLED
+static gboolean
+are_lomiri_snap_decisions_supported(IndicatorPowerNotifier * self)
+{
+  priv_t * const p = get_priv(self);
+
+  ensure_caps_queried(self);
+
+  return p->lomiri_snap_decisions_supported;
+}
+#endif
+
 static void
 notification_show(IndicatorPowerNotifier * self)
 {
@@ -309,7 +357,6 @@ notification_show(IndicatorPowerNotifier * self)
     icon_name = NULL;
   nn = notify_notification_new(title, body, icon_name);
   g_strfreev (icon_names);
-  g_free (body);
 
   if (are_actions_supported(self))
     {
@@ -330,11 +377,15 @@ notification_show(IndicatorPowerNotifier * self)
               notify_notification_set_hint(nn, "sound-file", g_variant_new_string("file://" LOMIRI_SOUNDSDIR "/notifications/" LOW_BATTERY_SOUND));
             }
         }
-
-      notify_notification_set_hint(nn, "x-lomiri-snap-decisions", g_variant_new_string("true"));
-      notify_notification_set_hint(nn, "x-lomiri-non-shaped-icon", g_variant_new_string("true"));
-      notify_notification_set_hint(nn, "x-lomiri-private-affirmative-tint", g_variant_new_string("true"));
-      notify_notification_set_hint(nn, "x-lomiri-snap-decisions-timeout", g_variant_new_int32(INT32_MAX));
+      #ifdef LOMIRI_FEATURES_ENABLED
+      if (are_lomiri_snap_decisions_supported(self)) {
+        /* Yes, all supposedly boolean values take strings... */
+        notify_notification_set_hint(nn, "x-lomiri-snap-decisions", g_variant_new_string("true"));
+        notify_notification_set_hint(nn, "x-lomiri-non-shaped-icon", g_variant_new_string("true"));
+        notify_notification_set_hint(nn, "x-lomiri-private-affirmative-tint", g_variant_new_string("true"));
+        notify_notification_set_hint(nn, "x-lomiri-snap-decisions-timeout", g_variant_new_int32(INT32_MAX));
+      }
+      #endif
       notify_notification_set_timeout(nn, NOTIFY_EXPIRES_NEVER);
       notify_notification_add_action(nn, "dismiss", _("OK"), on_dismiss_clicked, NULL, NULL);
       notify_notification_add_action(nn, "settings", _("Battery settings"), on_battery_settings_clicked, NULL, NULL);
@@ -355,6 +406,8 @@ notification_show(IndicatorPowerNotifier * self)
       g_error_free(error);
       g_object_unref(nn);
     }
+
+  g_free (body);
 }
 
 /***
@@ -369,6 +422,7 @@ on_battery_property_changed (IndicatorPowerNotifier * self)
   PowerLevel new_power_level;
   gboolean old_discharging;
   gboolean new_discharging;
+  gdouble new_percentage;
 
   g_return_if_fail(INDICATOR_IS_POWER_NOTIFIER(self));
   p = get_priv (self);
@@ -380,6 +434,8 @@ on_battery_property_changed (IndicatorPowerNotifier * self)
   old_discharging = p->discharging;
   new_discharging = indicator_power_device_get_state(p->battery) == UP_DEVICE_STATE_DISCHARGING;
 
+  new_percentage = get_battery_power_percentage (p->battery);
+
   /* pop up a 'low battery' notification if either:
      a) it's already discharging, and its PowerLevel worsens, OR
      b) it's already got a bad PowerLevel and its state becomes 'discharging */
@@ -394,6 +450,8 @@ on_battery_property_changed (IndicatorPowerNotifier * self)
     }
 
   dbus_battery_set_power_level (p->dbus_battery, power_level_to_dbus_string (new_power_level));
+  dbus_battery_set_power_percentage (p->dbus_battery, new_percentage);
+  dbus_battery_set_is_discharging (p->dbus_battery, new_discharging);
   p->power_level = new_power_level;
   p->discharging = new_discharging;
 }
